package main

import (
	"fmt"
	"log"
	"net/http"
	"os"
	"strings"
	"time"
)
//https://golang.org/doc/articles/wiki/#tmp_4
//Server that prints hello
// URL given to r, page data loaded.
// formatted HTML is then sent to the page via W

func urlSplitForCountry(a string)(string, string){

	// 0/1=conserv/2=v1/3=country/4=name/5=no&limit=10
	partsOne := strings.Split(a, "/")
	// 0=no & 1=limit=no
	partsTwo := strings.Split(partsOne[5], "&")
	// 0=&limit 1=10
	partsThree := strings.Split(partsOne[5],"=")

	CountryCode := partsTwo[0]
	fmt.Printf("CountryCode is: %s\n",CountryCode)

	Limit := partsThree[1]
    fmt.Printf("Limit is: %s\n",Limit)

	return CountryCode,Limit
}

func defaultHandler(w http.ResponseWriter, r *http.Request){
	//code to be added
}

func viewHandler(w http.ResponseWriter, r *http.Request) {

	//Code to do
	// should you test for client error or server error 400 / 500

	CountryCode, Limit := urlSplitForCountry(r.URL.Path)

	// Get country
	Country, Ccode, Flag := getCountryName(CountryCode)

	fmt.Fprintf(w,"Country name is %s\n Country code is: %s\n Flag is: %s\n ", Country, Ccode, Flag)

	// get species from a given country
	Species := getSpeciesByCountry(CountryCode,Limit)
	fmt.Fprintf(w,"Species %v\n",Species)
}

func speciesHandler(w http.ResponseWriter, r *http.Request){
	// 0/1=conserv/2=v1/3=species/4=speciesKey
	partSpecie:= strings.Split(r.URL.Path,  "/")
	speciesKey := partSpecie[4]

	// SpeciesKey from url
Species := getSpecies(speciesKey)

    // Year from url
Year:= getYearForSpecie(speciesKey)

    // write to web
fmt.Fprintf(w," Species: %v\n Year: %s\n ", Species, Year)

}

func diagnosticHandler(w http.ResponseWriter, r *http.Request){
	//  /conservation/v1/diag/
	StatGBIF, StatRestC, Version, Time := getDiagnostic()
	fmt.Fprintf(w," GBIF API Status: %d\n Rest Countries API Status: %d\n", StatGBIF, StatRestC)

	fmt.Fprintf(w," Version: %s\n UpTime: %s\n", Version, Time)

}
// Global var type time
var Start time.Time

// Main function
func main() {
	port := os.Getenv("PORT")
	
	if port == "" {
		//log.fatal formatting output and exits
		log.Fatal("$PORT must be set")
	}

	Start = time.Now()

    // we initialize http using viewHandler to handle any req
    // under the path / remember to end with trailing /

    // Add code to this function
	http.HandleFunc("/conservation/v1/", defaultHandler)

	http.HandleFunc("/conservation/v1/country/name/", viewHandler)

	http.HandleFunc("/conservation/v1/species/", speciesHandler)

	http.HandleFunc("/conservation/v1/diag/", diagnosticHandler)



    //LiAnSe listens on TCP net addr, calls serve with handler (nil)
    // -to handle req on incoming connections, if accept it keeps TCP alive
	log.Fatal(http.ListenAndServe(":"+port, nil))
}
