package main

import (
	"encoding/json"
	"fmt"
	"net/http"
	"time"
)

const apiRoot = "https://restcountries.eu/rest/v2/alpha/"

const apiRootSpeciesInCountry = "http://api.gbif.org/v1/occurrence/search?country="

const apiRootGBIF ="http://api.gbif.org/"

const apiRootRestCountr = "https://restcountries.eu/"

const limitSyntax = "&limit="

const version = "Version 1"

//Root for species by country request
const apiRootSpecies = "http://api.gbif.org/v1/species/"

//We receive JSON code and we parse it
//CntrInfo contains a response for restcountries endpoint
type CntrInfo struct {
	Flag string `json:flag`
	Name string `json:"name"` //"country":"norway"
	CountryCode string `json:"alpha2Code"`//"NO"

}

//Our json slice response
type CollectionFromReq struct {
	Count int `json:"count"`
	Collection []Result `json:"results"`
}

// Info about each specie
type Result struct {
	Species string `json:"species"`
	SpeciesKey int  `json:"speciesKey"`
}

// Info about each speciesKey
type SpeciesKeyBody struct{
	Key int `json:"specieskey"`
	Kingdom string `json:"kingdom"`
	Phylum string `json:"phylum"`
	Order string `json:"order"`
	Family string `json:"family"`
	Genus string `json:"genus"`
	ScientificName string `json:"scientificName"`
	CanonicalName string `json:"canonicalName"`
}

type SpeciesYear struct{
	//Hint: For the year, check the /species/{key}/name path for a given key.
	Year string `json:"bracketYear"`
}

func getDiagnostic() (int,int,string, time.Duration) {

	/*
	    "gbif": "<http status code for GBIF API>",
	   "restcountries": "<http status code for restcountries API>",
	   "version": "v1",
	   "uptime": <time in seconds from the last service restart>

	*/
	//res := Response{}
	urlGBIF := apiRootGBIF

	respDiag, err := http.Get(urlGBIF)
	if err !=nil {
		fmt.Println("Error:we have a problem", err)
		return 0,0, "",0
	}
	defer respDiag.Body.Close()
	//json.NewDecoder(respDiag.Body).Decode(ci)

	urlRestCountr := apiRootRestCountr
	respCountr, err := http.Get(urlRestCountr)
	if err !=nil {
		fmt.Println("Error:we have a problem", err)
		return 0, 0,"",0
	}
	defer respCountr.Body.Close()

	t := time.Now()
	elapsed := t.Sub(Start)



	return respDiag.StatusCode, respCountr.StatusCode, version, elapsed
}

//Should return the name Norway
func getCountryName(countryName string) (string, string, string) {

	// Instance of struct
	ci := &CntrInfo{}

    //Change hardcoded to func parameters "We pass them"
	//countryCode := "NO"
	//Implement passed parameters

	url := apiRoot + countryName
	fmt.Println("Dette er vår url",url)

	//resp is json, so we need to parse it
	//Sending get req to a given url
	resp, err := http.Get(url)

	fmt.Println("This is our response",resp)

	if err !=nil {
		fmt.Println("Error:we have a problem", err)
		return "", "",""
	}

	//Closes when surrounded functions are finished
	defer resp.Body.Close()


	//resp.Body JSON
	//NewDecoder returns a new decoder that reads from r
    json.NewDecoder(resp.Body).Decode(ci)
    fmt.Println("after json decoder",ci)

	return ci.Name, ci.CountryCode, ci.Flag
}

// Should return a struct with species and key that has occurrence in a pre defined country
func getSpeciesByCountry(CountryCode, LimitNumber string) CollectionFromReq {

    // One instance of struct
	cfr :=CollectionFromReq{}

	url := apiRootSpeciesInCountry + CountryCode + limitSyntax + LimitNumber

	fmt.Printf("This is limit number in getSpecByCountry function %s\n",LimitNumber)
	// Result stored as JSON in resp
	resp, err := http.Get(url)
	fmt.Printf("This is our url for species %s\n and this is our resp %s\n",url,resp)
	if err !=nil {
		fmt.Println("Error:we have a problem", err)
		return cfr
	}
	defer resp.Body.Close()

	// Takes resp.Body and decodes it into struct we have defined
	// Decoder will find the right name:pair even if its inside slice/array
	json.NewDecoder(resp.Body).Decode(&cfr) // array is inside result

	// returns struct
	return cfr

}


// takes speciesKey and returns struct
func getSpecies (SpeciesKey string) SpeciesKeyBody {

    skb:=SpeciesKeyBody{}


	url := apiRootSpecies + SpeciesKey


    respSpecies, err := http.Get(url)
	if err !=nil {
		fmt.Println("Error:we have a problem", err)
		return skb
	}
	defer respSpecies.Body.Close()

	json.NewDecoder(respSpecies.Body).Decode(&skb)
	return skb
}

func getYearForSpecie(SpeciesKey string) SpeciesYear {
	const Name = "name"
	sy:= SpeciesYear{}
	url:= apiRootSpecies + SpeciesKey + Name
	respYear, err:=http.Get(url)
	if err !=nil {
		fmt.Println("Error:we have a problem", err)
		return sy
	}
	defer respYear.Body.Close()
	json.NewDecoder(respYear.Body).Decode(&sy)
	return sy
}
